;
; load.asm - LOAD kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Load (A=verify, X,Y=load address)
;
; Used registers: A X Y (X,Y=next byte after end)
; Zeropage: $c3,$c4=X and Y register before call
; Zeropage: $ae,$af=next byte after end
; Status: C
; C=1:
; A=0 break
; A=4 file not found
; A=5 device not present
; A=8 missing filename
; A=9 illegal device number
; A=16 out of memory
; Status response:
; ST=$00 ok (only if run/stop was pressed)
; ST=$10 load/verify error (only if run/stop was pressed)
; ST=$40 load ok
; ST=$42 open error
; ST=$50 verify error
; ST=$52 load/verify error because of readerror (open error, output file, device...)
; ST=$80 device not present
;
; Notes:
; Secondary address 1..255 then load address from file is used
;**************************************************
;--------------------------------------
; LOAD
;
;		stx $c3
;		sty $c4
;		jmp ($0330)
;--------------------------------------
newload		.proc
		sta VerifyFlag		;Store verify flag (0=load, else verify)
		#stz_a Status		;Clear status
		stx $ae
		sty $af			;Store loading address ($c3,$c4 is the same...)
                
                .if DEBUGIO=1
                lda debug_io_exp
                beq kd
                #lda0
                sta debug_io_exp
                #psr $f4a5
                php
                lda VerifyFlag
                pha
                #phx_a
                #phy_a
                lda #3
                jmp monitor

kd		lda #1
		sta debug_io_exp
                .fi

		lda ActualDev
                beq kernal_load.ill
		#devicehandler hard_load,exit_illegal_device,pclink_load2,kernal_load
		.pend

		.if SUPERCPU=1
OP_restoredown  jsr OP_restore_zeropage
		jmp OP_scpu01down
		.else
OP_restoredown  = OP_restore_zeropage
		.fi

kernal_load	.proc
		tya
		beq ill
		cmp #31
		bcs ill
		cmp #3
		beq ill
		bcs ok
		#psr $f4b6
		tya
		jmp exit2basic		;Normal kernal rutin
ill		jmp exit_illegal_device

ok
		sta load_p_exp		;drive number
		jsr X_OP_load_common	;no pha!!

		#ldy_l_a load_p_exp     ;drive number
		lda ActualSecAddr
		sta load_limit_exp
		lda #$60
		sta ActualSecAddr

		jsr OP_cfg16up
		.if SERIAL_HANG_FIX=1
		jsr test_hang
		bcc np
		.fi
                tya
		jsr ieclisten		;listen
		lda #$f0
		jsr iecsecond		;second
		lda Status
		bpl present
np		jsr OP_cfg16down
devnotpre      	jsr OP_restoredown
		jmp exit_device_not_present
present
		ldx #0
cy		#sta_cfg_open
		lda path_exp,x
		#sta_cfg_16
		beq oy
		jsr iecout		;ciout
		inx
		bne cy
oy
		jsr iecunlisten		;unlsn

		tya
		jsr iectalk		;talk
		lda #$60
		jsr iectksa		;tksa
		jsr iecin
		tax
		lda Status
		and #2
		beq oe2
		jsr OP_cfg16down
                #jmps hard_load.fnf	;file not found
oe2
		jsr iecin
                pha
		lsr next_sector		;no fastload
		#sta_cfg_open
		lda load_limit_exp
		beq up
		stx $ae
                pla
		sta $af
                .byte SKIP1
up              pla

		lda #<$0333
		cmp $ae
		lda #>$0333
		sbc $af
		bcc above
                jsr OP_cfg16down

		ldy #2			;out of memory
		gne ex2
above

		.if (JIFFYDOS_ACCEL=0) & (SUPERCPU=0)
                lda JiffyDos_present_exp
                cmp #1
                .fi
		lda path_exp
		#sta_cfg_16
		eor #"$"
		beq uk

		.if (JIFFYDOS_ACCEL=0) & (SUPERCPU=0)
                bcc wr
                .fi

		bit Jiffyflag
                .if SUPERCPU=1
                bpl wr
                bvs wr
                .else
		bvc wr
                .fi
		jsr iecuntalk
                
		tya
		jsr iectalk
		lda #$61
		jsr iectksa
                .if (SUPERCPU=1) | (JIFFYDOS_ACCEL=0)
                sec
                ror next_sector
                .else
		asl Jiffyflag  		;fastload!
                .fi
                #jmps uk

wr		lda $01
                ora #7
                sta $01
		jsr fastloadinit

uk		jsr OP_cfg16down

		jsr OP_load_loading	;LOADING $xxxx-...

		#lda0
		ldx $af
		cpx #$d8		
		bcc fg			
		cpx #$dc		
		bcs fg
		lda #$dc
fg		sta load_top_exp                                    

;JiffyDOS SCPU Jaccel JDrive Fastload | Turboloader
;   0      0     0      x       0     |      0
;   0      0     0      x       1     |      1
;   0      0     1      0       0     |      0
;   0      0     1      0       1     |      1
;   0      0     1      1       x     |      1
;   x      1     x      0       0     |      0
;   x      1     x      0       1     |      1
;   x      1     x      1       x     |      1
;   1      0     x      0       0     |      0
;   1      0     x      0       1     |      1
;   1      0     x      1       x     |      1

		#OP_scpu01down

                lda next_sector
                .if (JIFFYDOS_ACCEL=1) & (SUPERCPU=0)
		ora Jiffyflag
                .fi
		bpl oldl

		sei
		.if SUPERCPU=1
                sta scpu_system_1mhz_start
                .fi
		lda $d015
		pha
		jsr jiffy_f0d8		;turn off sprites
		ldy $ae
		#lda0
		sta $ae
		geq tkm

nw		lda #1
		.if SUPERCPU=1
                sta scpu_system_1mhz_exit
                .fi
tkm		sta next_sector		;first
		#sta_cfg_open
		jsr OP_cfg16up
		jsr $f6bc		;check stop
		php
		jsr OP_cfg16down
		plp
		bpl stpp
		lda start_zeropage
		cmp #$60		;rts?
		beq stpp
		lda $d011
		and #$17
                cmp #$10
                bcs lc
		lda #$e1
lc      	adc #$1f-1
		sta next_sector+1
		lda $dd00
		and #7
		sta next_sector+2
		sta $dd00
		ora #$20
		tax
		bit $dd00
		bvc *-3
		bpl msc
		ldx #$64
kkw		bit $dd00
		bvc eof
		dex
		bne kkw
		lda #$42
		.byte SKIP2
eof		lda #$40
		ora Status
		sta Status
		clc
		.byte SKIP1
stpp		sec
		pla
		sta $d015
		sty $ae
                .if SUPERCPU=1
                ldy #15
                .else
		ldy #10
                .fi
                dey
                bne *-1
		bcc ex
		gcs sto
                
nw2		gvc nw

mmc		#sta_cfg_16
		.if SUPERCPU=1
                sta scpu_system_1mhz_start
                .fi
mjj		sec
mc		lda $d012
		sbc next_sector+1
		bcc mcx
		and #7
		beq mc
mcx		lda next_sector+2
		stx $dd00
		bit $dd00
		bvc nw2
		nop
		sta $dd00
		ora $dd00
		lsr
		lsr
		nop
		ora $dd00
		lsr
		lsr
		eor next_sector+2
		eor $dd00
		lsr
		lsr
		eor next_sector+2
		eor $dd00
		cpx #128
		bcc va
		sta ($ae),y
vav		iny
		bne mjj                
		.if SUPERCPU=1
                sta scpu_system_1mhz_exit
                .fi
		#sta_cfg_open
		inc $af
		lda $af
		cmp load_top_exp
		bne bc
		dec $af
                txa			;skip
                and #$7f
                tax
		lda #$60		;rts
		sta start_zeropage
bc		cmp #$d0
		beq mc2
		cmp #$e0
		bne mmc
		geq mc2

va		jsr start_zeropage
		sec
		gcs vav

msc		bit $dd00
		bpl *-3
		lda next_sector		;first?
		bne mmce		;no

mc2		#phx_a
		#phy_a
		jsr load_install_write_verify_byte3
		#ply_a
		pla
                and #$7f
		ldx start_zeropage+3
		cpx #$91		;sta ($nn),y
		bne nbv
		ora #$80
nbv		sta current_path

mmce		ldx current_path
		jmp mmc

oldl

loop2		jsr load_install_write_verify_byte3

		ldy #0
loop
		bit Status
		bvs ex
		jsr OP_cfg16up
		jsr iecin
		tax
		jsr OP_cfg16down
		txa
		jsr start_zeropage
		inc $ae
		bne loop
		jsr check_stop
		beq sto
		inc $af
		lda $af
		cmp load_top_exp
		beq out
		cmp #$d0
		beq loop2
		cmp #$e0
		bne loop
		geq loop2

out		iny
sto		iny
ex
		.if (JIFFYDOS_ACCEL=1) & (SUPERCPU=0)
		lda start_zeropage
		cmp #$60		;rts?
		bne noto
		lda load_top_exp
		sta $af
		#stz_a $ae
		ldy #2			;out of memory!
noto
		.fi
ex2
		lda load_p_exp		;drive number
		pha
		#OP_scpu01up
		jsr OP_cfg16up
		jsr iecuntalk		;untalk
		pla
		jsr ieclisten		;f648
		lda #$e0
		jsr iecsecond
		jsr iecunlisten
		jsr OP_cfg16down
                tya
                #jmps hard_load.handleerr
		.pend
;-----------------------------------
; Install zeropage routine for kernal loader
;
load_install_write_verify_byte3 .proc
                ldy #verify_byte_r3-write_byte_r3+14
		lda VerifyFlag
		bne u
                ldy #write_byte_r3-write_byte_r3+14
		lda $af
		cmp #$d0
		bcc u
		cmp #$e0
		bcs u
		lda load_top_exp
		bne u
                ldy #write_io_r3-write_byte_r3+14
u		ldx #14
vsd		lda write_byte_r3,y
		sta start_zeropage,x
                dey
		dex
		bpl vsd
		rts
		.pend
;-----------------------------------
; Write a byte (for load)
;
write_byte_r3	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
		sta ($ae),y		;2
		sta cfg_open		;3
		rts			;1
		.here
		.pend
;-----------------------------------
; Verify a byte (for load)
;
verify_byte_r3	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
		cmp ($ae),y		;2 5
		sta cfg_open		;3
		beq *+6			;2 5 15
		lda #$10		;2
		sta Status		;2
		rts			;1
		.here
		.pend
;-----------------------------------
; Verify a byte (for load)
;
write_io_r3	.proc
		.logical start_zeropage
                jmp write_byte_io_r3
                .byte $4c
		.here
		.pend
;-----------------------------------
; loading bytes under I/O
;
write_byte_io_r3 .proc
		#sta_cfg_16
		php
		pha
		lda #2
		sei
		sta $1
		pla
		sta ($ae),y
		lda #6
		plp
		sta $1
		jmp OP_cfg16down
		.pend
;--------------------------------------
; LOAD from PCLink
;--------------------------------------
pclink_load2	.proc                
		jsr X_OP_load_common

		jsr load_install_write_verify_byte3

		jsr pclink_load
		bcc load_end
                dey
                bmi hard_load.outmem
                tya
                bne hard_load.fnf
		geq kernal_load.devnotpre
		.pend
;--------------------------------------
; LOAD from IDE
;--------------------------------------
hard_load	.proc
		jsr X_OP_load_common

		ldy #0
		tsx
fg		cpx #$03+52		;enough stack space?
		bcs ook
		pla
		sta sp_exp,x		;no, let's backup
		iny
		inx
		gne fg
ook		#phy_a

		jsr XIT_load
		sta pom1
		sty pom2

		#ply_a
		beq kb
		ldx #$03+52
nk		dex
		lda sp_exp,x		;restore stack
		pha
		dey
		bne nk
kb
		ldy pom2
		lda pom1
		bcc handleerr
		#ldatax current_drive_exp
		lda status_pos_exp,x
		cmp #$80+39
		beq fnf
		cmp #$80+62
		beq fnf
		lda #$52		;Error... (load error)
		sta Status
handleerr       tax
		dex
		beq brek		;1 break
		dex
		bne load_end		;2 outmem

outmem		jsr OP_restoredown
		jmp exit_out_of_memory

brek		jsr OP_restoredown
		#lda0
		sec
		jmp exit2basic

fnf		jsr OP_restoredown
		jmp exit_file_not_found

load_end	.proc
		jsr OP_restore_zeropage
		tya
		clc
		adc $ae
		sta $ae
		bcc *+4
		inc $af
		lda load_autostart2_exp	;autostart?
		beq tp
		bmi jmpp
		ldx #4
		stx 198
ne		lda run-1,x
		sta 630,x
		dex
		bne ne
		geq tp

jmpp		lda load_autostart_start_exp
		sec
		sbc #1
		tax
		lda load_autostart_start_exp+1
		sbc #0
		pha
		#phx_a

tp		#lda0
		sta load_autostart_exp	;no autostart
		lda Status
		ora #%01000000
		sta Status
		and #%10101111
		bne *+5
		jsr OP_load_loading_end
		#OP_scpu01down
		ldx $ae
		ldy $af
		clc
		jmp exit2basic
run		.text "R","U"+128,":",13

XIT_load	.proc
		#xit_init

		jsr check_drive		;check drive is alive

		jsr XIT_init_drive	;-=> error

		lda path_exp
		cmp #"$"
		bne XIT_load_file       ;LOAD"$"?
;--------------------------------------
; LOAD directory
;
XIT_load_dir	.proc
		ldx #19
vsd		lda write_byte_r,x
		ldy VerifyFlag
		beq cbd
		lda verify_byte_r,x
cbd		sta start_zeropage,x
		dex
		bpl vsd

		ldx ActualSecAddr	;Load to specifyed address?
		beq jkl
		lda #1
		sta $ae
		lda #4			;Load address from file ($0401)
		sta $af
jkl
		lda $ae
		sta start_zeropage+5
		cmp #<$0334
		lda $af
		sta start_zeropage+6
		sbc #>$0334
		bcc mem
                cmp #$cf-$03-1
                bcs mem

		lda #1*2
		jsr XIT_dir_first	;Init dir stuff

		jsr OP_load_loading	;LOADING $xxxx-...
load_dirlp	jsr check_stop
		beq stop

		lda #$cf
		jsr XIT_dirlp2		;-=> error
                lda start_zeropage+4
                and #$20
                bne mem
		lda direntry_first_exp
		bpl load_dirlp

		ldy start_zeropage+5
		#lda0
		sta $ae
		.byte SKIP2
stop		lda #1			;-=> break
		.byte SKIP2
mem		lda #2			;-=> out of memory
		ldx start_zeropage+6
		stx $af
		clc
		rts			;-=> ok
		.pend
;-----------------------------------
; LOAD file
;
XIT_load_file	.proc
		lda #RESOLV_LOAD_MODE
		jsr OP_XIT_resolv_path_part;-=> file not found

		lda resolv_anything_exp
		beq aj			;LOAD CHR$(0),12

                lda direntry_mode_exp
                cmp #DIRMODE_CFS
                beq load_cfs
                cmp #DIRMODE_ISO
                beq load_iso
                cmp #DIRMODE_JOLIET
                beq load_iso
aj      	sec
                gcs LWOP_reporterror_62_xit;only iso/cfs is supported
                
load_iso	jsr X_check_load_s        ;Adjust filesize
		bcc abs2

		lda #<2049
                ldy #>2049
                jsr cd_limit_read

		.byte SKIP2
abs2		lda #1			;1 sector
                sta load_sectors_exp
		pha

		jsr XIT_upload_next_sector;-=> error
		#ply_a
		jsr OP_cd_read
		jsr LWOP_reporterror_read_xit;-=> error
		lda #ram_exp_buffer0
		jsr OP_read2048_here

		jsr check_abs		;no pha!!!

		ldx #ver_lot_iso_re-ver_lot_iso_r-1
                ldy #ver_lot_iso_re-load_lot_iso_r-1

		lda VerifyFlag
		bne vsd2

                ldx #load_lot_iso_re-load_lot_iso_r-1
                .if SUPERCPU=1
                txy
                .else
                ldy #load_lot_iso_re-load_lot_iso_r-1
                .fi
		#psr back2+1

vsd2		lda load_lot_iso_r,y
		pha
                dey
		dex
		bpl vsd2

		lda load_size	;file smaller than 2046 bytes?
		sec
		sbc #<2046
		tax
		lda load_size+1
		sbc #>2046
                ldy #2
		bcc copyrestofbuffer
		sta load_size+1

		lda #>2046
                jsr loadflush_start2

		clc

		tsx
		lda $ae
		sta load_lot_iso_r.at1+1,x

		ldy VerifyFlag
		beq lll2

		sta load_lot_iso_r.at2+1+2,x
		lda $af
		sta load_lot_iso_r.at2+2+2,x
		gcc be2x

lll2		sta load_lot_iso_r.at2+1,x
		lda $af
		sta load_lot_iso_r.at2+2,x
be2x		sta load_lot_iso_r.at1+2,x
		gcc be2

vback2		.if SUPERCPU=1          ;Verify for ISO
		php			
		jsr OP_scpu01up
                plp
                .fi
		beq jo2
		lda #$10
		sta Status
ly2             iny
		beq jo2
                lda data_g
		iny
                beq jo2
                gne ly2

tnt2
		jsr XIT_upload_next_sector;-=> error

                jsr cd_limit_read2047
                tay
		jsr OP_cd_read
		bcs handle_error_

		tsx
		lda status_g
		lda low_reg
read_loop2	and #%00001000
		beq handle_error2_	;DRQ=1?

		lda load_size+1
		sec
		sbc #8
		bcc lessthan2048
		sta load_size+1

		lda #8
                sta pom1

loop		ldy #0
		lda VerifyFlag
		bne we2
		lda load_lot_iso_r.at1+2,x
		cmp #$cf
		bcc we2
		cmp #$e0
		bcc load_lot_io_iso_r
we2
		sec
		#OP_scpu01down
		lda #1
		pha
		#phx_a
		rts

handle_error_	#jmps handle_error
handle_error2_	#jmps handle_error2

back2		#OP_scpu01up

jo2     	
	    	tsx
		inc load_lot_iso_r.at1+2,x

		ldy VerifyFlag
		beq llo3
                inc load_lot_iso_r.at2+2+2,x
                #jmps llo4
llo3    	inc load_lot_iso_r.at2+2,x
llo4
		dec pom1
                bne loop

be2     	lda status_g
		lda low_reg
		bpl okbsy2
		jsr wait_busy
		bcs handle_error_
		lda status_g
		lda low_reg
		tsx
okbsy2
		#dec_l load_sectors_exp
		bne read_loop2

		and #%00001000
		bne handle_error2_	;DRQ=0?

		lda load_size
		ora load_size+1
		bne tnt2
		tay
                lda load_lot_iso_r.at1+2,x
		sta $af			;$ae is correct...
		#jmps eof

lessthan2048    lda #ram_exp_buffer0
		jsr OP_read2048_here

		tsx
		lda load_lot_iso_r.at1+2,x
		#jmps copyrestofbuffer2
;------------------------
load_cfs
		lda #ram_exp_tree	;Load tree sector
		jsr XIT_read_a_sector	;-=> error
                
                jsr X_check_load_s	;Adjust filesize
		bcc abs

		jsr savesize
		lda load_size
		adc #2-1
		sta load_size	;2 more bytes for first lincalc
                bcc kqk
                inc load_size+1
kqk

		ldy #ram_exp_tree
		jsr linearaccess

		jsr restoresize
		#jmps rel1

abs		lda #1
		sta load_sectors_exp	;1 sector loaded
		sta load_p_exp		;from sector 1

		#cnext ram_exp_tree_exp , next_sector;first sector address

rel1		
		jsr XIT_upload_next_sector;-=> error
		#ldy_l_a load_sectors_exp;1 sector
		jsr OP_read
		jsr LWOP_reporterror_read_xit;-=> error
		ldx #ram_exp_buffer0
		jsr read_buffer_cfs_safe

		jsr check_abs		;no pha!!!

		ldx #ver_lot_cfs_re-ver_lot_cfs_r-1
                ldy #ver_lot_cfs_re-load_lot_cfs_r-1

		lda VerifyFlag
		bne vsd

                ldx #load_lot_cfs_re-load_lot_cfs_r-1
                .if SUPERCPU=1
                txy
                .else
                ldy #load_lot_cfs_re-load_lot_cfs_r-1
                .fi
		#psr back+1

vsd		lda load_lot_cfs_r,y
		pha
                dey
		dex
		bpl vsd

		lda load_size	;file smaller than 510 bytes?
		sec
		sbc #<510
		tax
		lda load_size+1
		sbc #>510
	       	ldy #2
		bcc copyrestofbuffer
		sta load_size+1

		lda #>510
                jsr loadflush_start2

		clc

		tsx
		lda $ae
		sta load_lot_cfs_r.at1+1,x
                dec $af

		ldy VerifyFlag
		beq lll

		sta load_lot_cfs_r.at3+1+4,x
		adc #$80
		sta load_lot_cfs_r.at2+1+2,x
		sta load_lot_cfs_r.at4+1+6,x
		lda $af
		sta load_lot_cfs_r.at1+2,x
		adc #255
		sta load_lot_cfs_r.at2+2+2,x
		clc
		gcc bex

lll		sta load_lot_cfs_r.at3+1,x
		adc #$80
		sta load_lot_cfs_r.at2+1,x
		sta load_lot_cfs_r.at4+1,x
		lda $af
		sta load_lot_cfs_r.at1+2,x
		adc #255
		sta load_lot_cfs_r.at2+2,x
		clc
bex		gcc be

vback		#OP_scpu01up		;Verify for CFS
		iny
		dey
		bmi jo
		lda #$10
		sta Status
		bcc ly
		lda data_g		;skip the rest
ly		iny
		bmi jo
		lda data_g		;skip the rest
		lda data_g		;skip the rest
		gcs ly

tnt
		ldy #ram_exp_tree
		jsr linearaccess

		jsr XIT_upload_next_sector;-=> error

		#ldy_l_a load_sectors_exp;1 sector
		jsr OP_read
		bcs handle_error

		tsx
		lda status_g
		lda low_reg
read_loop	and #%00001000
		beq handle_error2	;DRQ=1?

		lda load_size+1
		sec
		sbc #2
		bcc lessthan512
		sta load_size+1

		ldy #0
		lda VerifyFlag
		bne we
		lda load_lot_cfs_r.at1+2,x
		cmp #$ce
		bcc we
		cmp #$e0
		bcc load_lot_io_cfs_r
we
		sec
		#OP_scpu01down
		lda #1
		pha
		#phx_a
		rts

back		#OP_scpu01up

jo		sec
		tsx
be      	lda load_lot_cfs_r.at1+2,x
		adc #1
		sta load_lot_cfs_r.at1+2,x
		#adc1

		ldy VerifyFlag
		beq llo
		sta load_lot_cfs_r.at3+2+4,x
		lda load_lot_cfs_r.at2+2+2,x
		adc #2
		sta load_lot_cfs_r.at2+2+2,x
		#adc1
		sta load_lot_cfs_r.at4+2+6,x
		jmp llo2

handle_error2	sec
		lda #EUNEXP
handle_error	jmp LWOP_reporterror_read_xit

llo		sta load_lot_cfs_r.at3+2,x
		lda load_lot_cfs_r.at2+2,x
		adc #2
		sta load_lot_cfs_r.at2+2,x
		#adc1
		sta load_lot_cfs_r.at4+2,x

llo2
		lda status_g
		lda low_reg
		bpl okbsy
		jsr wait_busy
		bcs handle_error
		lda status_g
		lda low_reg
		tsx
okbsy
		#dec_l load_sectors_exp
		bne read_loop

		and #%00001000
		bne handle_error2	;DRQ=0?

		lda load_size
		ora load_size+1
		bne tnt
		tay
                lda load_lot_cfs_r.at1+2,x
		sta $af			;$ae is correct...
		#jmps eof

lessthan512	ldx #ram_exp_buffer0
		jsr read_buffer_cfs_safe

		tsx
		lda load_lot_cfs_r.at1+2,x
copyrestofbuffer2
		sta $af			;$ae is correct...
                ldy #0
copyrestofbuffer
		jsr ata_pi_nomore_data
		bcs handle_error

                ldx load_size
		lda load_size+1
                jsr loadflush_start

eof		#xit_exit

		clc
		lda load_overflow_exp
		bmi nov
		lda load_top_exp
		sta $af
		#stz_a $ae
		lda #2			;out of memory
                .byte SKIPLDA0
nov		#lda0
		rts			;-=> ok

load_lot_io_cfs_r
		jsr savesize
		lda load_lot_cfs_r.at4+2,x
		sta pom5
		lda load_lot_cfs_r.at3+2,x
		sta pom7
		lda load_lot_cfs_r.at2+2,x
		sta pom9
		lda load_lot_cfs_r.at1+2,x
		sta pom11
		lda load_lot_cfs_r.at4+1,x
		sta pom4
		sta pom8
		lda load_lot_cfs_r.at3+1,x
		sta pom6
		sta pom10
		jsr OP_cfg16up
                .if SUPERCPU=0
		php
		pla
		and #%00000100
		beq lp			;sei?

lp2		ldx #2
		lda data_g
		lda low_reg
		pha
		lda hig_reg
		pha
		lda data_g
		lda low_reg
;                .if SUPERCPU=1
;                xba
;                .else
		sta pom12
;                .fi
		lda hig_reg
		stx $1
		sta (pom4),y
;                .if SUPERCPU=1
;                xba
;                .else
		lda pom12
;                .fi
		sta (pom6),y
		pla
		sta (pom8),y
		pla
		sta (pom10),y
		ldx #6
		stx $1

		iny
		bpl lp2
		gmi lw
                .fi

lp		ldx #2
		lda data_g
                .if SUPERCPU=1
                php
                .fi
		lda low_reg
		pha
		lda hig_reg
		pha
		lda data_g
		lda low_reg
                .if SUPERCPU=1
                xba
                .else
		sta pom12
                .fi
		lda hig_reg
		sei
		stx $1
		sta (pom4),y
                .if SUPERCPU=1
                xba
                .else
		lda pom12
                .fi
		sta (pom6),y
		pla
		sta (pom8),y
		pla
		sta (pom10),y
		ldx #6
                .if SUPERCPU=1
                plp
                .else
		cli
                .fi
		stx $1

		iny
		bpl lp

lw		jsr OP_cfg16down
		jsr restoresize
		jmp jo

load_lot_io_iso_r
		lda load_lot_cfs_r.at1+1,x
		sta pom4
		sta pom6
		lda load_lot_cfs_r.at1+2,x
		sta pom5
		sta pom7
		jsr OP_cfg16up
                .if SUPERCPU=0
		php
		pla
		and #%00000100
		beq lpc 		;sei?

lp2c    	ldx #2
		lda data_g
		lda hig_reg
;                .if SUPERCPU=1
;                xba
;                .else
		sta pom12
;                .fi
		lda low_reg
		stx $1
		sta (pom4),y
                iny
;                .if SUPERCPU=1
;                xba
;                .else
		lda pom12
;                .fi
		sta (pom6),y
		ldx #6
		stx $1

		iny
		bne lp2c
		gmi lwc
                .fi

lpc     	ldx #2
		lda data_g
                .if SUPERCPU=1
                php
                .fi
		lda hig_reg
                .if SUPERCPU=1
                xba
                .else
		sta pom12
                .fi
		lda low_reg
		sei
		stx $1
		sta (pom4),y
                iny
                .if SUPERCPU=1
                xba
                .else
		lda pom12
                .fi
		sta (pom6),y
		ldx #6
                .if SUPERCPU=1
		plp
                .else
                cli
                .fi
		stx $1

		iny
		bne lpc

lwc     	jsr OP_cfg16down
		jmp jo2
		.pend
;----------------------------------
;Flush start of buffer
;                
loadflush_start2 .proc
		stx load_size
		ldx #$fe
loadflush_start	.proc
                sta pom4
                sty pom1
                
		ldy #8
vsd2		lda load_1_cfs_r,y
		sta start_zeropage,y
		dey
		bpl vsd2
		ldy VerifyFlag
                beq kok
                lda #$51		;eor
                sta load_1_cfs_r.at
kok		
		.if SUPERCPU=1
		ldy #1
		sty pom3
                dey
                .else
                ldy #0
                .fi

		lda #ram_exp_buffer0
		sta pom2
                txa
                beq nu

nin		jsr load_verify_part
		tya
                beq nin2
		clc
		adc $ae			;adjust destination address
		sta $ae
		bcc koko
		inc $af
                clc
koko    	tya
		ldy #0
                adc pom1
                sta pom1
                bcc koko2
                .byte SKIP2
nin2            inc $af
                inc pom2
koko2
nu              dec pom4
                bpl nin
kkk             rts
		.pend
;-----------------------------------
; Load/verify part of the buffer
;
load_verify_part .proc
		lda VerifyFlag
		bne ry5c
		lda $af
		cmp #$cf
		bcc ry5
		cmp #$e0
		bcs ry5
ry4		php
		lda $1
                pha
		#lda_zp_y pom1
		pha
		lda #2
		sei
                #sta_cfg_16
		sta $1
		pla
		sta ($ae),y
		pla
		sta $1
                plp
		#sta_cfg_open
		iny
		dex
		bne ry4
		rts

ry5		#lda_zp_y pom1
		jsr start_zeropage
		iny
		dex
		bne ry5
		rts

ry5c		#lda_zp_y pom1
		jsr start_zeropage
		beq dcc
		lda #$10
		sta Status
dcc		iny
		dex
		bne ry5c
		rts
		.pend
;-----------------------------------
; CFS load/verifying 1 byte normal Z=0 if no match (for verify_part)
;
load_1_cfs_r	.proc
		.logical start_zeropage
		sta cfg_stnd
at		sta ($ae),y		;or eor
		sta cfg_open
		rts
                .here
                .pend
;-----------------------------------
; Calculate number of bytes to load to load_size
; C=1 if loading below $0334
;
load_calc_needed .proc
		lda $ae
		cmp #<$0334
		lda $af
		sbc #>$0334
		bcs above
		rts

above		ldy #>$0000		;memory end

		lda direntry_f_size_exp
		sbc #2
		sta load_size
		lda direntry_f_size_exp+1
		sbc #0
		bcs nt			;file shorter than 2 bytes???
		tya
		sta load_size
nt		sta load_size+1

		ldx $af			;loading limits ($10000/$dc00)
		cpx #>$d800
		bcc nl1
		cpx #>$dc00
		bcs nl1
		ldy #>$dc00		;memory end
nl1
		#lda0
		sec
		sbc $ae
		tax
		tya			;Calculate space until limit
		sta load_top_exp
		sbc $af
		tay

		cpx load_size
		sbc load_size+1
		bcs xxe		;space is bigger than file...
		sty load_size+1
		stx load_size
xxe
		.if SUPERCPU=1
		ror
		sta load_overflow_exp
		.else
		ror load_overflow_exp	;if overflow<128 than file was bigger than available space
		.fi
		sec
		rts
		.pend
;-----------------------------------
; CFS load 512 bytes normal
;
load_lot_cfs_r	.proc
		.logical $101
		sta cfg_stnd		;3
lp		lda data_g		;6

		lda low_reg		;9
at1		sta $0000,y		;12

		lda hig_reg		;15
at2		sta $0000,y		;18

		lda data_g		;21

		lda low_reg		;24
at3		sta $0000,y		;27

		lda hig_reg		;30
at4		sta $0000,y		;33

		iny			;34

		bpl lp			;36
		sta cfg_open		;39
		.byte $4c		;40
		.here
		.pend
load_lot_cfs_re
;-----------------------------------
; CFS verifying 512 bytes normal Z=0 if no match
;
ver_lot_cfs_r	.proc
		.logical $101
		sta cfg_stnd		;3
lp		lda data_g		;6

		lda low_reg		;9
at1		eor $0000,y		;12
		bne err			;14
		lda hig_reg		;17
at2		eor $0000,y		;20
		bne err			;22

		lda data_g		;25

		lda low_reg		;28
at3		eor $0000,y		;31
		bne err2		;33
		lda hig_reg		;36
at4		eor $0000,y		;39
		bne err2		;41

		iny			;42

		bpl lp			;44
		.byte $a9		;45
err2		clc			;46
err
		sta cfg_open		;49
		jmp XIT_load_file.vback	;52
		.here
		.pend
ver_lot_cfs_re
;-----------------------------------
; ISO load 256 bytes normal
;
load_lot_iso_r  .proc
		.logical $101
		sta cfg_stnd		;3
lp		lda data_g		;6

		lda low_reg		;9
at1		sta $0000,y		;12

		iny			;13

		lda hig_reg		;16
at2		sta $0000,y		;19

		iny			;20

		bne lp			;22
		sta cfg_open		;25
		.byte $4c		;26
		.here
		.pend
load_lot_iso_re
;-----------------------------------
; ISO verifying 256 bytes normal Z=0 if no match
;
ver_lot_iso_r	.proc
		.logical $101
		sta cfg_stnd		;3
lp		lda data_g		;6

		lda low_reg		;9
at1_		eor $0000,y		;12
		bne err			;14
                
                iny
                
		lda hig_reg		;17
at2_		eor $0000,y		;20
		bne err			;22

		iny			;42

		bne lp			;44
err
		sta cfg_open		;49
		jmp XIT_load_file.vback2;52
		.here
		.pend
ver_lot_iso_re
;-----------------------------------
; Write a byte (for directory)
;
write_byte_r	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
lp		pla			;4
		sta $0000,y		;7
		dey			;8
		bpl lp			;10
		sta cfg_open		;13
		rts			;14
		.here
		.pend
;-----------------------------------
; Verify a byte (for directory)
;
verify_byte_r	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
lp		pla			;4
		eor $0000,y		;7
		beq *+6			;9
		lda #$10		;11
		sta Status		;13
		dey			;14
		bpl lp			;16
		sta cfg_open		;19
		rts			;20
		.here
		.pend


;----------------------------------
;Print saving/searching
;
OP_save_saving	.proc
		bit msgmode
		bpl nomess
		jsr OP_cfg16up
		ldy #$51		; "SAVING "
		jsr OP_load_save_name
		jsr OP_cfg16up
		lda #32
		jsr newchrout_t
		lda #"$"
		jsr newchrout_t
		lda $c2
		jsr NO_printhx
		lda $c1
		jsr NO_printhx
		jmp OP_cfg16down
nomess		rts
		.pend

OP_load_searching_for .proc
		bit msgmode
		bpl nomess
		jsr OP_cfg16up
		ldy #$0c		; "SEARCHING "
		jsr $f12f
		ldy #$17		; "FOR "

OP_load_save_name .proc
		jsr $f12f
		ldx #0
kqu		#sta_cfg_open
		lda path_exp,x
		beq jkxe
		jsr newchrout_t
		inx
		bne kqu
jkxe		jmp OP_cfg16down
		.pend
nomess		rts
;----------------------------------
;Print loading/verifying $xxxx-$xxxx
;
OP_load_loading_end .proc
		ldx #"-"
		sec
		.byte SKIP1
OP_load_loading	.proc
		clc
		lda $ae
		sta load_autostart_start_exp
		lda $af
		sta load_autostart_start_exp+1
		bit msgmode
		bpl nomess
		jsr OP_cfg16up
		bcs bkjxd
		jsr $f5d2		;"LOADING"/"VERIFYING"
		ldx #$20
bkjxd		txa
		jsr newchrout_t
		lda #"$"
		jsr newchrout_t
		lda $af
		jsr NO_printhx
		lda $ae
		jsr NO_printhx
htnoq		jmp OP_cfg16down
		.pend

;----------------------------------
;Common start for load
;
X_OP_load_common .proc
		lda ActualFNLength
		beq miss

		#OP_scpu01up
		jsr OP_save_zeropage
		jsr OP_copy_filename_to_path
		jsr OP_load_searching_for
		jmp OP_loadautos

miss		pla
		pla
                jmp exit_missing_filename
                .pend
;----------------------------------
;Check load size
;
X_check_load_s 	.proc
		lda direntry_f_size_exp+2
		ora direntry_f_size_exp+3;file bigger than 65535 bytes?
		beq notoobig
		lda #255
;		sta direntry_f_size_exp	;65535 will also cause OUT OF MEMORY...
		sta direntry_f_size_exp+1
notoobig
		clc
		lda ActualSecAddr
                bne vwv2
					;load to (X/Y)
		sta load_p_exp		;from sector 0

		jsr load_calc_needed
		bcs vwv2
                pla
                pla
		lda #2                  ;-=> out of memory
vwv2            rts
		.pend
;----------------------------------
;Update pointer
;
OP_cd_read	.proc
		tya
		clc
                adc next_sector+3
                sta next_sector+3
                bcc cu
                ldx #2
li              inc next_sector,x
		bne cu
                dex
                bpl li
cu      	jmp OP_read
		.pend
;----------------------------------
;Limit sector transfer
;
cd_limit_read2047 .proc
		lda #<2047
                ldy #>2047
cd_limit_read	.proc
		clc
                adc load_size
		#ldatax current_drive_exp
		tya
		adc load_size+1
                ror
                lsr
                lsr
		cmp setup_linear_r_hdd0_exp,x
		bcc idi
		beq idi
		#lda0			;lda #1-1
		adc setup_linear_r_hdd0_exp,x
idi		sta read_sectors_exp
		rts
		.pend
;----------------------------------
;Check absolute
;
check_abs	.proc
		ldx ActualSecAddr	;Load to (X/Y)?
		beq rel2
		lda load_address_exp	;no
		sta $ae
		lda load_address_exp+1
		sta $af

		jsr load_calc_needed
		bcs rel2
		jsr ata_pi_nomore_data
		jsr LWOP_reporterror_read_xit;-=> error
                pla
                pla
		lda #2
		rts			;-=> out of memory

rel2 		#jmps OP_load_loading
		.pend
